import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A cat which likes pizzas and really hates bombs
 * 
 * @author Jan Hartman
 * @version 1.0. 5.7.2022
 */
public class Cat extends Actor
{
    private int speed; // how fast the cat moves
    
    private boolean eating; // whether the cat is in the middle of eating
    private int eatingTime; // how many cycles the cat spends eating always
    private int eatCount; // how many cycles the cat has been eating
    
    /**
     * Creates a new cat with its eating time limit and initial speed
     */
    public Cat(){
        eatingTime = 20;
        speed = 5;
        eating = false;
    }
    
    /**
     * Makes the cat move, catch and eat pizzas and explode
     * when it hits a bomb
     */
    public void act()
    {
        checkPizza();
        if (eating)
        {
            continueEating();
        }
        else if (Greenfoot.isKeyDown("right") && !Greenfoot.isKeyDown("left"))
        {
            walk(speed);
        }
        else if (Greenfoot.isKeyDown("left") && !Greenfoot.isKeyDown("right"))
        {
            walk(-speed);
        }
        else 
        {
            setImage("cat.png");
        }
        checkBomb();
    }
    
    /**
     * Makes the cat walk. Negative speed means left.
     */
    private void walk(int speed)
    {
        setLocation(getX()+speed, getY());

        if (speed < 0){
            animate("cat-walk-left");
        }
        else {
            animate("cat-walk-right");
        }
    }
    
    /**
     * Makes the cat continue eating and takes care of the animation
     */
    private void continueEating()
    {
        eatCount++;
        animate("cat-eat");
        if (eatCount > eatingTime)
        {
            eating = false;
            eatCount = 0;
        }
        
    }
    
    /**
     * Checks, whether there is a pizza nearby and starts eating it if yes
     */
    private void checkPizza()
    {
        Pizza pizza = (Pizza) getOneIntersectingObject(Pizza.class);
        if (pizza != null){
            CatWorld catworld = (CatWorld) getWorld();
            catworld.removeObject(pizza);
            eating = true;
            eatCount = 0;
            catworld.addEatenPizza();
            Greenfoot.playSound("slurp.wav");
        }
    }
    
    /**
     * Checks, whether there is a bomb and makes the cat explode if yes
     */
    private void checkBomb()
    {
        Bomb bomb = (Bomb) getOneIntersectingObject(Bomb.class);
        if (bomb != null){
            CatWorld catworld = (CatWorld) getWorld();
            catworld.removeObject(bomb);
            catworld.addObject(new Explosion(),getX(),getY());
            catworld.removeObject(this);
            catworld.setCatDead();
        }
    }
    
    /**
     * Creates an animation effect of two well named pictures (0 and 1 at end)
     * if invoked every act cycle.
     */
    private void animate(String root)
    {
        CatWorld world = (CatWorld) getWorld();
        if (world.getTime() % 3 == 0){
            setImage(root + "-" + world.getTime() % 2 + ".png");

        }
    }
    
}
