import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A cat world where pizzas and bombs fall from the sky
 * 
 * @author Jan Hartman 
 * @version 1.0 5.7.2022
 */
public class CatWorld extends World
{
    private int time; // total time the game has been running for
    private int eatenPizzas; // total number of pizzas eaten by cat
    private int timeLimit; // how long the game lasts
    
    private int loseTimeDelay; // how long after dying scoreboard shows
    private boolean catAlive; // is the cat alive?
    /**
     * Creates a new world with time counter, total pizza counter
     * and eaten pizza counter
     */
    public CatWorld()
    {    
        super(600, 400, 1);
        prepare();
        catAlive = true;
        time = 0;
        timeLimit = 1000;
        loseTimeDelay = 20;
        eatenPizzas = 0;
        showTime();
        showEatenPizzas();
    }
    
    /**
     * Makes the world count time and generate pizzas and bomb
     */
    public void act()
    {
        countTime();
        generatePizza();
        generateBomb();
    }
    
    /**
     * Returns current time
     */
    public int getTime()
    {
        return time;
    }
    
    /**
     * Increases the eaten pizza count
     */
    public void addEatenPizza()
    {
        eatenPizzas++;
        showEatenPizzas();
    }
    
    /**
     * Generates pizzas at the top of the screen randomly
     */
    private void generatePizza()
    {
        if (Greenfoot.getRandomNumber(100) < 1){
            Pizza pizza = new Pizza(Greenfoot.getRandomNumber(2)+2);
            addObject(pizza, Greenfoot.getRandomNumber(600), 20);
        }
    }
    
    /**
     * Generates bombs at the top of the screen randomly
     */
    private void generateBomb()
    {
        if (Greenfoot.getRandomNumber(1000) < 10){
            Bomb bomb = new Bomb(Greenfoot.getRandomNumber(3)+3);
            addObject(bomb, Greenfoot.getRandomNumber(600),20);
        }
    }
    
    /**
     * Counts track of current time
     */
    private void countTime()
    {
        time++;
        showTime();
        if (time > timeLimit)
        {
            if (catAlive) {
                showWinBoard();
                GreenfootSound winSound = new GreenfootSound("game-win.wav");
                winSound.setVolume(70);
                winSound.play();
            }
            else {
                showLoseBoard();
                Greenfoot.playSound("game-over.wav");
            }
            Greenfoot.stop();
        }
    }
    
    /**
     * Writes a text showing the number of pizzas eaten by the cat 
     */
    private void showEatenPizzas()
    {
        showText("Score: " + eatenPizzas, 500, 25);
    }
    
    /**
     * Writes a text showing the current time
     */
    private void showTime()
    {
        showText("Time: " + time, 100, 25);
    }
    
    /**
     * Places the cat at the bottom of the screen in the middle of the world
     */
    private void prepare()
    {
        Cat cat = new Cat();
        addObject(cat, 300, 340);
    }
    
    private void showWinBoard()
    {
        ScoreBoard board = new ScoreBoard("You Won!","Score: ",eatenPizzas);
        addObject(board, 300,200);
    }
    
    public void setCatDead()
    {
        catAlive = false;
        timeLimit = time + loseTimeDelay;
    }
    
    private void showLoseBoard()
    {
        ScoreBoard board = new ScoreBoard("You Lost!","Score: ",eatenPizzas);
        addObject(board, 300,200);
    }
}
